import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper
from thales_remote.file_interface import ThalesFileInterface

from zahner_analysis.file_import.isc_import import IscImport

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

fileInterface = ThalesFileInterface("localhost")
fileInterface.disableSaveReceivedFilesToDisk()
fileInterface.enableKeepReceivedFilesInObject()
fileInterface.enableAutomaticFileExchange(fileExtensions="*.isc")

zahnerZennium.setCVStartPotential(0)
zahnerZennium.setCVUpperReversingPotential(0.2)
zahnerZennium.setCVLowerReversingPotential(-0.2)
zahnerZennium.setCVEndPotential(0)

zahnerZennium.setCVStartHoldTime(2)
zahnerZennium.setCVEndHoldTime(2)

zahnerZennium.setCVCycles(1.5)
zahnerZennium.setCVSamplesPerCycle(400)

zahnerZennium.setCVMaximumCurrent(0.0002)
zahnerZennium.setCVMinimumCurrent(-0.0002)

zahnerZennium.setCVOhmicDrop(0)

zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
zahnerZennium.disableCVAnalogFunctionGenerator()

zahnerZennium.setCVNaming("individual")
zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

scanRatesForMeasurement = [0.5, 1, 2]

for scanRate in scanRatesForMeasurement:
    zahnerZennium.setCVOutputFileName("cv_{:d}mVs".format(int(scanRate * 1000)))
    zahnerZennium.setCVScanRate(scanRate)

    zahnerZennium.checkCVSetup()

    zahnerZennium.measureCV()

    """
    Determine the maximum current for the next measurement
    from the maximum current of the last CV measurement.
    """
    latestMeasurement = IscImport(fileInterface.getLatestReceivedFile().binaryData)
    maximumCurrent = max(abs(latestMeasurement.getCurrentArray()))

    zahnerZennium.setCVMaximumCurrent(maximumCurrent * 3)
    zahnerZennium.setCVMinimumCurrent(maximumCurrent * -3)

    zenniumConnection.disconnectFromTerm()
    fileInterface.close()

    iscFileFromDisc = IscImport(r"C:\THALES\temp\cv\cv_1000mVs.isc")

    iscFiles = [IscImport(file.binaryData) for file in fileInterface.getReceivedFiles()]

    for iscFile in iscFiles:
        print(
            f"{iscFile.getScanRate()} V/s\tmeasurement finished at {iscFile.getMeasurementEndDateTime()}"
        )


figCV, (axis) = plt.subplots(1, 1)
figCV.suptitle("Cyclic Voltammetry at different scan rates")

for iscFile in iscFiles:
    axis.plot(
        iscFile.getVoltageArray(),
        iscFile.getCurrentArray(),
        label=f"{iscFile.getScanRate()} $\\frac{{V}}{{s}}$",
    )

axis.grid(which="both")
axis.xaxis.set_major_formatter(EngFormatter(unit="$V$"))
axis.yaxis.set_major_formatter(EngFormatter(unit="$A$"))
axis.set_xlabel(r"$Voltage$")
axis.set_ylabel(r"$Current$")
axis.legend()

figCV.set_size_inches(18, 18)
plt.show()
figCV.savefig("CV.svg")



from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
from thales_remote.error import TermConnectionError, ThalesRemoteError

from zahner_potentiostat.scpi_control.searcher import SCPIDeviceSearcher
from zahner_potentiostat.scpi_control.serial_interface import (
    SerialCommandInterface,
    SerialDataInterface,
)
from zahner_potentiostat.scpi_control.control import *
from zahner_potentiostat.scpi_control.datahandler import DataManager

import threading
import time
from dataclasses import dataclass


class EpcScpiHandler:
    r"""Class for the control objects.

    This class manages the object composed of a cennium and the external potentiostat.
    The object contains an instance of a :class:`~zahner_potentiostat.scpi_control.control.SCPIDevice`
    and the shared common :class:`~thales_remote.script_wrapper.ThalesRemoteScriptWrapper` object.

    The SCPI object is invalid when the device is in EPC mode.
    The device must be manually switched from EPC to SCPI before the SCPI object can be used again.

    :param sharedZennium: Zennium object.
    :type sharedZennium: :class:`~thales_remote.script_wrapper.ThalesRemoteScriptWrapper`
    :param epcChannel: Number of the EPC channel to which the device is connected via EPC cable.
        If a Rmux card is plugged in then the numbers have an offset.
    :param serialNumber: Serial number of the external potentiostat.
    """

    zenniumMutex = threading.Lock()  # class variable
    sharedZenniumInterface: ThalesRemoteScriptWrapper
    _epcId: int
    _serialNumber: int
    _isInEPC: bool
    _commandInterface: Union[SerialCommandInterface, None]
    scpiInterface: Union[SCPIDevice, None]

    def __init__(
        self,
        sharedZennium: ThalesRemoteScriptWrapper,
        epcPotentiostatId: int,
        serialNumber: int,
    ):
        self.sharedZenniumInterface = sharedZennium
        self._epcId = epcPotentiostatId
        self._serialNumber = serialNumber
        self._isInEPC = True

        self._commandInterface = None
        self.scpiInterface = None
        return

    def isSharedZenniumAvailable(self) -> bool:
        r"""Check if the zennium is available.

        The method checks if the threading.lock for synchronizing access to the Zennium is available.

        :returns: True if the zennium is not locked and available.
        """
        return EpcScpiHandler.zenniumMutex.locked() == False

    def acquireSharedZennium(self, blocking: bool = True, timeout: int = -1) -> bool:
        r"""Check if the Zennium is available.

        Wrapper for the aquire method of the `Python lock object <https://docs.python.org/3/library/threading.html#lock-objects>`_.
        The parameters and return values are simply passed through.

        :param blocking: When invoked with the blocking argument set to True (the default),
            block until the lock is unlocked, then set it to locked and return True.
        :param timeout: When invoked with the floating-point timeout argument set to a positive value,
            block for at most the number of seconds specified by timeout and as long as the lock cannot
            be acquired. A timeout argument of -1 specifies an unbounded wait. It is forbidden to
            specify a timeout when blocking is false.
        :returns: The return value is True if the lock is acquired successfully, False if not
            (for example if the timeout expired).
        """
        return EpcScpiHandler.zenniumMutex.acquire(blocking, timeout)

    def releaseSharedZennium(self) -> None:
        r"""Release the Zennium object.

        Wrapper for the aquire method of the `Python lock object <https://docs.python.org/3/library/threading.html#lock-objects>`_.

        Release a lock. This can be called from any thread, not only the thread which has acquired the lock.
        When the lock is locked, reset it to unlocked, and return. If any other threads are blocked
        waiting for the lock to become unlocked, allow exactly one of them to proceed.

        When invoked on an unlocked lock, a RuntimeError is raised.
        """
        EpcScpiHandler.zenniumMutex.release()
        return

    def connectSCPIDevice(self) -> None:
        r"""Establish connection to the potentiostat.

        This method establishes the connection to the potentiostat (PP2x2, XPOT2 and EL1002) and passes it to the internal data structure.
        When invoked on an unlocked lock, a RuntimeError is raised.
        """
        deviceSearcher = SCPIDeviceSearcher()
        deviceSearcher.searchZahnerDevices()
        commandSerial, dataSerial = deviceSearcher.selectDevice(self._serialNumber)
        self._commandInterface = SerialCommandInterface(commandSerial)

        self.scpiInterface = SCPIDevice(
            self._commandInterface, SerialDataInterface(dataSerial)
        )
        return

    def switchToSCPIAndReleaseSharedZennium(
        self, keepPotentiostatState: bool = False
    ) -> None:
        r"""Switch from EPC to SCPI mode of the potentiostat and release the Zennium.

        The switch from EPC to SCPI must be made from the EPC operation, both control options can
        only release control but cannot take control away from each other.

        After the control is released, the Zennium is released.

        :param keepPotentiostatState: If this parameter is True,
            the potentiostat is not switched off when switching from EPC to SCPI.
        """
        self.sharedZenniumInterface.selectPotentiostat(self._epcId)

        if keepPotentiostatState:
            self.sharedZenniumInterface.switchToSCPIControlWithoutPotentiostatStateChange()
        else:
            self.sharedZenniumInterface.switchToSCPIControl()

        self._isInEPC = False
        self.releaseSharedZennium()

        """
        It takes some time for the operating system to recognize the USB device.
        
        It tries to find the USB device 3 times every 3 seconds. If this fails, an exception is thrown.
        """
        maxTry = 3
        for i in range(maxTry):
            try:
                time.sleep(3)
                self.connectSCPIDevice()
            except Exception as e:
                if i == (maxTry - 1):
                    raise e
            else:
                break  # for loop
        return

    def switchToSCPI(self, keepPotentiostatState: bool = False) -> None:
        r"""Switch from EPC to SCPI mode.

        It is recommended to use :class:`~thales_remote.epc_scpi_handler.EpcScpiHandler.switchToSCPIAndReleaseSharedZennium` instead of this function.

        Before calling this method, the Zennium must have been released, since these methods call
        aquire and release themselves.
        If the Zennium was locked before this function will block.

        :param keepPotentiostatState: If this parameter is True,
            the potentiostat is not switched off when switching from EPC to SCPI.
        """
        self.acquireSharedZennium()

        if keepPotentiostatState:
            self.sharedZenniumInterface.selectPotentiostatWithoutPotentiostatStateChange(
                self._epcId
            )
            self.sharedZenniumInterface.switchToSCPIControlWithoutPotentiostatStateChange()
        else:
            self.sharedZenniumInterface.selectPotentiostat(self._epcId)
            self.sharedZenniumInterface.switchToSCPIControl()

        self._isInEPC = False
        self.releaseSharedZennium()
        """
        Wait a little so that windows recognizes the new usb device when the potentiostat logs on again.
        """
        time.sleep(3)
        self.connectSCPIDevice()
        return

    def switchToEPC(self, keepPotentiostatState: bool = False) -> None:
        r"""Switch from SCPI to EPC mode.

        Before calling this method the Zennium must be locked with aquire.

        This method is used to switch from SCPI to EPC operation. After this method is called, the
        scpiInterface object is destroyed because the USB connection is closed.

        This method automatically selects the correct EPC channel.

        :param keepPotentiostatState: If this parameter is True,
            the potentiostat is not switched off when switching from SCPI to EPC.
        """
        try:
            if keepPotentiostatState:
                self.scpiInterface.switchToEPCControlWithoutPotentiostatStateChange()
            else:
                self.scpiInterface.switchToEPCControl()
            self.scpiInterface.close()
        except:
            pass
        finally:
            self.scpiInterface = None
            self._isInEPC = True

        """
        Wait a little for the change to EPC.
        """
        time.sleep(2)
        if keepPotentiostatState:
            self.sharedZenniumInterface.selectPotentiostatWithoutPotentiostatStateChange(
                self._epcId
            )
        else:
            self.sharedZenniumInterface.selectPotentiostat(self._epcId)
        return

    def getSerialNumber(self):
        return self._serialNumber

    def close(self):
        r"""Close the SCPI connection.

        The function is not required in epc mode.
        """
        if self._isInEPC is False:
            self.scpiInterface.close()
        return


@dataclass
class HandlerDataItem:
    serialNumber: int
    epcIndex: int
    handlerObject: EpcScpiHandler


class EpcScpiHandlerFactory:
    r"""Class for creating the control objects.

    This class initializes the connection to the zennium.
    The :func:`~epc_scpi_handler.EpcScpiHandlerFactory.createEpcScpiHandler` method can then be used
    to create a control object for the corresponding device.

    :param shared_zennium_target: IP address at which the Zennium can be reached. Default is "localhost".
    """

    _zenniumConnection: ThalesRemoteConnection
    sharedZenniumInterface: ThalesRemoteScriptWrapper
    _handlerList: list[HandlerDataItem]

    def __init__(self, shared_zennium_target="127.0.0.1"):
        self._zenniumConnection = ThalesRemoteConnection()
        connectionSuccessful = self._zenniumConnection.connectToTerm(
            shared_zennium_target, "ScriptRemote"
        )
        if connectionSuccessful is False:
            raise TermConnectionError("connection to zennium not possible")

        self.sharedZenniumInterface = ThalesRemoteScriptWrapper(self._zenniumConnection)
        self.sharedZenniumInterface.forceThalesIntoRemoteScript()
        self._handlerList = []
        return

    def getSharedZennium(self) -> ThalesRemoteScriptWrapper:
        r"""Returns the zennium object.

        Returns the Zennium object, which contains the Remote2 commands as methods.

        :returns: Object with the Remote2 wrapper.
        """
        return self.sharedZenniumInterface

    def getZenniumConnection(self) -> ThalesRemoteConnection:
        r"""Returns the zennium connection object.

        Returns the object that manages the connection to the zennium.

        :returns: Object with the connection to the zennium.
        """
        return self._zenniumConnection

    def createEpcScpiHandler(
        self, epcChannel: int, serialNumber: int
    ) -> EpcScpiHandler:
        r"""Returns the zennium connection object.

        This method initializes the external potentiostats and creates the objects.

        The objects are in SCPI mode after calling this function.
        For compatibility, the devices always start in EPC mode when connected to EPC, then they must
        be switched to SCPI standalone mode via Remote2. It is only possible to switch to SCPI mode via Remote2.

        :param epcChannel: Number of the EPC channel to which the device is connected via EPC cable.
            If a Rmux card is plugged in then the numbers have an offset.
        :param serialNumber: Serial number of the external potentiostat.
        :returns: Object with the external potentiostat.
        """
        newDevice = EpcScpiHandler(self.getSharedZennium(), epcChannel, serialNumber)

        deviceSearcher = SCPIDeviceSearcher()
        deviceSearcher.searchZahnerDevices()
        commandSerial: SerialCommandInterface = None
        dataSerial: SerialDataInterface = None

        try:
            commandSerial, dataSerial = deviceSearcher.selectDevice(serialNumber)
        except:
            pass

        """
        If the device is not found, then it is checked whether it is found as an EPC device.
        If it is found as an EPC device, it is switched to SCPI mode.
        """
        if commandSerial is None and dataSerial is None:
            newDevice.acquireSharedZennium()
            newDevice.sharedZenniumInterface.selectPotentiostat(epcChannel)
            name, serial = newDevice.sharedZenniumInterface.getDeviceInformation()
            newDevice.releaseSharedZennium()

            if serial not in str(serialNumber):
                raise ThalesRemoteError("Potentiostat is not found on the EPC channel.")

            newDevice.switchToSCPI()
        else:
            newDevice.connectSCPIDevice()

        listItem = HandlerDataItem(serialNumber, epcChannel, newDevice)

        self._handlerList.append(listItem)

        return newDevice

    def closeAll(self) -> None:
        r"""Close connections to all devices.

        This command closes all connections to the external potentiostats and to the Zennium.
        """
        for element in self._handlerList:
            element.handlerObject.close()

        self._handlerList = []

        self._zenniumConnection.disconnectFromTerm()
        return



import sys
import os

script_path = os.path.dirname(os.path.realpath(__file__))
if script_path not in sys.path:
    sys.path.append(script_path)

from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper


class ZenniumSingleton:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def init_zennium(self, target="localhost", connectionName="ScriptRemote"):
        self.zenniumConnection = ThalesRemoteConnection()
        self.zenniumConnection.connectToTerm(target, connectionName)
        self.zahnerZennium = ThalesRemoteScriptWrapper(self.zenniumConnection)
        self.zahnerZennium.forceThalesIntoRemoteScript()
        return

    def disconnect(self):
        return self.zenniumConnection.disconnectFromTerm()


zennium_instance = ZenniumSingleton()


def init(target="localhost", connectionName="ScriptRemote"):
    r"""
    This function connects to the Term Software and creates the ThalesRemoteScriptWrapper object.

    :param target: IP address of the computer on which the term is running, defaults to "localhost".
    :param connectionName: Name of the connection, defaults to "ScriptRemote".
    """
    global zennium_instance
    zennium_instance.init_zennium(target, connectionName)
    return


def disconnect():
    r"""
    Disconnect the connection.
    """
    global zennium_instance
    zennium_instance.disconnect()
    return


def _generate_function(method_name):
    r"""
    Helper function that creates functions from the object's methods.
    """

    def function(*args, **kwargs):
        zennium_method = getattr(zennium_instance.zahnerZennium, method_name)
        return zennium_method(*args, **kwargs)

    return function


def transformMethodsToFunctions():
    r"""
    After init() is called, the transformMethodsToFunctions() function must be called.

    init() creates the ThalesRemoteScriptWrapper object and transformMethodsToFunctions() converts the methods of this object into functions.
    Since the LabVIEW Python node can only execute functions from Python and not methods from objects.

    The functions are then in the global Python scope. This means that the same LabVIEW session must be used for all functions.

    Useful links from the LabVIEW documentation:

    * https://www.ni.com/docs/en-US/bundle/labview-api-ref/page/functions/python-node.html
    * https://knowledge.ni.com/KnowledgeArticleDetails?id=kA00Z0000019UFmSAM&l=en-US
    """
    for method_name in dir(zennium_instance.zahnerZennium):
        if not method_name.startswith("_") and method_name not in globals():
            globals()[method_name] = _generate_function(method_name)
    return



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper

from zahner_analysis.file_import.ism_import import IsmImport
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter, StrMethodFormatter
from matplotlib import ticker, colors, cm

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    zahnerZennium.setEISNaming("individual")
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setLowerFrequencyLimit(100)
    zahnerZennium.setStartFrequency(100)
    zahnerZennium.setUpperFrequencyLimit(500000)
    zahnerZennium.setLowerNumberOfPeriods(10)
    zahnerZennium.setLowerStepsPerDecade(10)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(10)
    zahnerZennium.setScanDirection("startToMin")
    zahnerZennium.setScanStrategy("single")

    potentialsToMeasure = np.linspace(0, 0.3, 13)
    print(potentialsToMeasure)

    for potential in potentialsToMeasure:
        filename = "{:d}_mvdc".format(int(round(potential * 1000)))
        print("step: " + filename)
        zahnerZennium.setEISOutputFileName(filename)
        zahnerZennium.setPotential(potential)

        zahnerZennium.enablePotentiostat()
        zahnerZennium.measureEIS()
        zahnerZennium.disablePotentiostat()

    zahnerZennium.setAmplitude(0)
    zenniumConnection.disconnectFromTerm()

    absoluteImpedances = []
    phases = []

    for potential in potentialsToMeasure:
        ismFile = IsmImport(
            r"C:\THALES\temp\{:d}_mvdc.ism".format(int(round(potential * 1000)))
        )
        absoluteImpedances.append(ismFile.getImpedanceArray())
        phases.append(ismFile.getPhaseArray())

    absoluteImpedances = np.array(absoluteImpedances)
    phases = np.array(phases)
    phases = np.abs(phases * (360 / (2 * np.pi)))

    impedanceFrequencies = ismFile.getFrequencyArray()

    X, Y = np.meshgrid(impedanceFrequencies, potentialsToMeasure)

    impedanceFigure, impedancePlot = plt.subplots(1, 1)
    impedanceFigure.suptitle("Impedance vs. DC Voltage vs. Frequency")

    ticks = np.power(
        10,
        np.arange(
            np.floor(np.log10(absoluteImpedances.min()) - 1),
            np.ceil(np.log10(absoluteImpedances.max()) + 1),
        ),
    )
    levels = np.logspace(
        np.floor(np.log10(absoluteImpedances.min()) - 1),
        np.ceil(np.log10(absoluteImpedances.max())),
        num=200,
    )
    impedanceContour = impedancePlot.contourf(
        X,
        Y,
        absoluteImpedances,
        levels=levels,
        norm=colors.LogNorm(absoluteImpedances.min(), absoluteImpedances.max(), True),
        cmap="jet",
    )

    impedancePlot.set_xlabel(r"Frequency")
    impedancePlot.set_xscale("log")
    impedancePlot.xaxis.set_major_formatter(EngFormatter(unit="Hz"))

    impedancePlot.set_ylabel(r"DC Voltage")
    impedancePlot.yaxis.set_major_formatter(EngFormatter(unit="V"))

    impedanceBar = impedanceFigure.colorbar(
        impedanceContour, ticks=ticks, format=EngFormatter(unit="$\Omega$")
    )
    impedanceBar.set_label("| Impedance |")
    impedanceFigure.set_size_inches(14, 12)
    plt.tight_layout()
    plt.show()
    impedanceFigure.savefig("impedance_contour.svg")

    phaseFigure, phasePlot = plt.subplots(1, 1)
    phaseFigure.suptitle("Phase vs. DC Voltage vs. Frequency")

    levels = np.linspace(phases.min(), phases.max(), 91)
    phaseContour = phasePlot.contourf(X, Y, phases, levels=levels, cmap="jet")

    phasePlot.set_xlabel(r"Frequency")
    phasePlot.set_xscale("log")
    phasePlot.xaxis.set_major_formatter(EngFormatter(unit="Hz"))

    phasePlot.set_ylabel(r"DC Voltage")
    phasePlot.yaxis.set_major_formatter(EngFormatter(unit="V"))

    phaseBar = phaseFigure.colorbar(
        phaseContour, format=StrMethodFormatter("{x:.0f}$°$")
    )
    phaseBar.set_label("| Phase |")

    phaseFigure.set_size_inches(14, 12)
    plt.tight_layout()
    plt.show()
    phaseFigure.savefig("phase_contour.svg")



import os

from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper, PotentiostatMode
from thales_remote.file_interface import ThalesFileInterface

from zahner_analysis.analysis_tools.eis_fitting import EisFitting, EisFittingPlotter
from zahner_analysis.plotting.impedance_plot import bodePlotter
from zahner_analysis.file_import.impedance_model_import import IsfxModelImport
from zahner_analysis.file_import.isc_import import IscImport
from zahner_analysis.file_import.ism_import import IsmImport

import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np
from matplotlib.ticker import EngFormatter
import jinja2

remoteIP = "localhost"

zenniumConnection = ThalesRemoteConnection()
zenniumConnection.connectToTerm(remoteIP)

zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
zahnerZennium.forceThalesIntoRemoteScript()

measDataInterface = ThalesFileInterface(remoteIP)
measDataInterface.disableSaveReceivedFilesToDisk()
measDataInterface.enableKeepReceivedFilesInObject()
measDataInterface.enableAutomaticFileExchange(fileExtensions="*.ism*.isc")

zahnerZennium.calibrateOffsets()

zahnerZennium.setCVStartPotential(0)
zahnerZennium.setCVUpperReversingPotential(1)
zahnerZennium.setCVLowerReversingPotential(0)
zahnerZennium.setCVEndPotential(0)

zahnerZennium.setCVStartHoldTime(2)
zahnerZennium.setCVEndHoldTime(2)

zahnerZennium.setCVCycles(1.5)
zahnerZennium.setCVSamplesPerCycle(400)

zahnerZennium.setCVMaximumCurrent(1e-3)
zahnerZennium.setCVMinimumCurrent(-1e-3)

zahnerZennium.setCVOhmicDrop(0)

zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
zahnerZennium.disableCVAnalogFunctionGenerator()

zahnerZennium.setCVNaming("individual")
zahnerZennium.setCVOutputPath(r"C:\THALES\temp")
zahnerZennium.setCVOutputFileName("cv_measurement")

scanRate = 0.25

zahnerZennium.setCVScanRate(scanRate)

zahnerZennium.checkCVSetup()
zahnerZennium.measureCV()

CVmeasurementData = IscImport(measDataInterface.getLatestReceivedFile().binaryData)
maximumCurrent = max(CVmeasurementData.getCurrentArray())
minimumCurrent = min(CVmeasurementData.getCurrentArray())

capacitanceCV = ((maximumCurrent - minimumCurrent) / 2) / scanRate

capacitanceFormatter = EngFormatter(places=3, unit="F", sep="")
print(f"Capacitance CV: {capacitanceFormatter.format_data(capacitanceCV)}")

figCV, (axis) = plt.subplots(1, 1)

axis.plot(
    CVmeasurementData.getVoltageArray(),
    CVmeasurementData.getCurrentArray(),
    color="red",
)

axis.grid(which="both")
axis.xaxis.set_major_formatter(EngFormatter(unit="V"))
axis.yaxis.set_major_formatter(EngFormatter(unit="A"))
axis.set_xlabel(r"Voltage")
axis.set_ylabel(r"Current")

figCV.set_size_inches(10, 10)
plt.show()
figCV.savefig("CV.pdf")

zahnerZennium.setEISNaming("individual")
zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
zahnerZennium.setEISOutputFileName("eis_measurement")

zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
zahnerZennium.setAmplitude(10e-3)
zahnerZennium.setPotential(1)
zahnerZennium.setLowerFrequencyLimit(5)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(500000)
zahnerZennium.setLowerNumberOfPeriods(3)
zahnerZennium.setLowerStepsPerDecade(3)
zahnerZennium.setUpperNumberOfPeriods(20)
zahnerZennium.setUpperStepsPerDecade(5)
zahnerZennium.setScanDirection("startToMin")
zahnerZennium.setScanStrategy("single")

zahnerZennium.enablePotentiostat()
zahnerZennium.measureEIS()
zahnerZennium.disablePotentiostat()

zahnerZennium.setAmplitude(0)

EISmeasurementData = IsmImport(measDataInterface.getLatestReceivedFile().binaryData)

zenniumConnection.disconnectFromTerm()
measDataInterface.close()

impedanceCircuitModel = IsfxModelImport("rlc-model.isfx")
print("fit start values:")
print(impedanceCircuitModel)

fitting = EisFitting()

fitParams = {
    "UpperFrequencyLimit": 200e3,
    "DataSource": "zhit",  # "original", "smoothed" or "zhit"
    "Smoothness": 0.0002,
}

simulationParams = {
    "UpperFrequencyLimit": 10e6,
    "LowerFrequencyLimit": 500e-3,
    "NumberOfSamples": 150,
}

simulatedData = fitting.simulate(impedanceCircuitModel, simulationParams)

(fig1, (impedanceAxis1, phaseAxis1)) = bodePlotter(impedanceObject=EISmeasurementData)
fig1.set_size_inches(10, 5)
fig1.savefig("EIS.pdf")

(fig1, (impedanceAxis1, phaseAxis1)) = bodePlotter(
    (impedanceAxis1, phaseAxis1),
    impedanceObject=simulatedData,
    argsImpedanceAxis={"linestyle": "solid", "marker": None},
    argsPhaseAxis={"linestyle": "solid", "marker": None},
)

impedanceAxis1.legend(
    impedanceAxis1.get_lines() + phaseAxis1.get_lines(),
    2 * ["Measured Data", "Model start values"],
)
fig1.set_size_inches(10, 5)
plt.show()

fittingResult = fitting.fit(
    impedanceCircuitModel,
    EISmeasurementData,
    fitParams=fitParams,
    simulationParams=simulationParams,
)

capacitanceEIS = fittingResult.getFittedModel()["C0"]["C"].getValue()
print(f"Capacitance EIS: {capacitanceFormatter.format_data(capacitanceEIS)}")

print(fittingResult)

(fig2, (impedanceAxis2, phaseAxis2)) = EisFittingPlotter.plotBode(
    fittingResult, EISmeasurementData
)
impedanceAxis2.legend(
    impedanceAxis2.get_lines() + phaseAxis2.get_lines(),
    2 * ["Measured Data", "Fitted Model"],
    loc="lower left",  # with capacitors, there are never measurement data at this position
)
fig2.set_size_inches(10, 5)
plt.show()

fig2.savefig("EIS_fitted.pdf")

mpl.rcParams["axes.unicode_minus"] = False
prefixFormatter = EngFormatter(places=3, sep="")
defaultMu = prefixFormatter.ENG_PREFIXES[-6]
prefixFormatter.ENG_PREFIXES[-6] = "\\textmu"  # LaTeX notation for micro

with open("EIS.csv", "wb") as file:
    file.write(bytearray("Frequency;Impedance;Phase" + os.linesep, "utf-8"))

    for freq, imp, phase in zip(
        EISmeasurementData.getFrequencyArray(),
        EISmeasurementData.getImpedanceArray(),
        EISmeasurementData.getPhaseArray(),
    ):
        file.write(
            bytearray(
                f"{prefixFormatter.format_data(freq)};{prefixFormatter.format_data(imp)};{prefixFormatter.format_data(phase * (360 / (2 * np.pi)))}"
                + os.linesep,
                "utf-8",
            )
        )

objectName = input("Input Test Object Identifier:")

EISmeasurementData.save(f"{objectName}.ism")

fittingResult.save(
    path=f"{objectName}_result",
    exist_ok=True,
    saveFitResultJson=True,
    saveFittedModel=True,
    saveFittedSimulatedSamples=True,
    saveFitInputSamples=True,
    fitResultJsonFilename="fit_result.json",
    fittedModelFilename="fitted.isfx",
    fittedSimulatedDataFilename="fitted_simulated.ism",
    fitInputDataFilename="fit_samples.ism",
)

latex_jinja_env = jinja2.Environment(
    variable_start_string="\PYVAR{",
    variable_end_string="}",
    trim_blocks=True,
    autoescape=False,
    loader=jinja2.FileSystemLoader(os.path.abspath(".")),
)

template = latex_jinja_env.get_template(r"report.tex")

currentFormatter = EngFormatter(places=3, unit="A", sep="")
currentFormatter.ENG_PREFIXES[-6] = "\\mu "  # LaTeX notation for micro
resistanceFormatter = EngFormatter(places=3, unit="$\Omega$", sep="")
resistanceFormatter.ENG_PREFIXES[-6] = "\\mu "
inductanceFormatter = EngFormatter(places=3, unit="H", sep="")
inductanceFormatter.ENG_PREFIXES[-6] = "\\mu "

fittedModel = fittingResult.getFittedModel()
fitResult = fittingResult.getFitResultJson()

fileString = template.render(
    objectname=objectName,
    capacitance_cv=capacitanceFormatter.format_data(capacitanceCV),
    cv_filename="CV.pdf",
    eis_filename="EIS.pdf",
    eis_fitted_filename="EIS_fitted.pdf",
    eis_csv_filename="EIS.csv",
    cv_maximum_current=currentFormatter.format_data(maximumCurrent),
    cv_minimum_current=currentFormatter.format_data(minimumCurrent),
    cv_scanrate=scanRate,
    capacitance_eis=capacitanceFormatter.format_data(capacitanceEIS),
    capacitance_eis_error=f"{fitResult['model']['C0']['C']['error']:.1f}",
    resistance_eis=resistanceFormatter.format_data(fittedModel["R0"]["R"].getValue()),
    resistance_eis_error=f"{fitResult['model']['R0']['R']['error']:.1f}",
    inductance_eis=inductanceFormatter.format_data(fittedModel["L0"]["L"].getValue()),
    inductance_eis_error=f"{fitResult['model']['L0']['L']['error']:.1f}",
)

fileString = bytearray(fileString, "utf-8")
f = open("report_filled.tex", "wb")
f.write(fileString)
f.close()

# only needed for Jupyter, that the kernel does not have to be restarted all the time.
prefixFormatter.ENG_PREFIXES[-6] = defaultMu
currentFormatter.ENG_PREFIXES[-6] = defaultMu
resistanceFormatter.ENG_PREFIXES[-6] = defaultMu
inductanceFormatter.ENG_PREFIXES[-6] = defaultMu

# close pdf viewer so that the pdf can be overwritten for debugging
os.system("taskkill /IM Acrobat.exe /T /F")

command = f'pdflatex.exe report_filled.tex -jobname="{objectName}"'
os.system(command)
os.popen(f"{objectName}.pdf")
print("finish")



r"""
  ____       __                        __    __   __      _ __
 /_  / ___ _/ /  ___  ___ ___________ / /__ / /__/ /_____(_) /__
  / /_/ _ `/ _ \/ _ \/ -_) __/___/ -_) / -_)  '_/ __/ __/ /  '_/
 /___/\_,_/_//_/_//_/\__/_/      \__/_/\__/_/\_\\__/_/ /_/_/\_\

Copyright 2024 Zahner-Elektrik GmbH & Co. KG

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


class ThalesRemoteError(Exception):
    r"""Thales Remote Exception Class

    This exception is thrown when an error is reported in the remote protocol, for example,
    when a parameter is out of range.

    The error numbers can be found in the Remote2 manual in chapter 6 from page 37:

    * Link without page number:
        * https://doc.zahner.de/manuals/remote2.pdf
    * Link with page number (may not work with every browser/pdf reader):
        * https://doc.zahner.de/manuals/remote2.pdf#page=37

    The following is an example for troubleshooting when an exception is thrown.

    **Example exception**

    .. code-block:: bash

        Traceback (most recent call last):
        File "C:\XXX\Thales-Remote-Python\ie_example.py", line 62, in <module>
            zahnerZennium.setIESecondEdgePotential(10000)
        File "C:\XXX\Thales-Remote-Python\ThalesRemote\ThalesRemoteScriptWrapper.py", line 648, in setIESecondEdgePotential
            return self.setValue("IE_EckPot2", potential)
        File "C:\XXX\Thales-Remote-Python\ThalesRemote\ThalesRemoteScriptWrapper.py", line 1026, in setValue
            raise ThalesRemoteError(reply.rstrip("\r") + ThalesRemoteScriptWrapper.undefindedStandardErrorString)
        ThalesRemoteError.ThalesRemoteError: ERROR;100;1

    **Explanation**

    In the last line you can read what kind of error has occurred:

    .. code-block:: bash

        ThalesRemoteError.ThalesRemoteError: ERROR;100;1
            ^                                       ^
            |                                       |
            |        Error number from the Remote2 manual https://doc.zahner.de/manuals/remote2.pdf
            |        You can read in the table in chapter 6 of the manual:
            |        100 | ERROR_PARAMETER_OUT_OF_RANGE | Sent value too low/high
            |
            ThalesRemoteError shows that it is an error generated by the library due to a response from the Zenniums containing an error.

    The first two lines of the traceback show the file and line number and the content of the line where the error occurred.

    .. code-block:: bash

        File "C:\XXX\Thales-Remote-Python\ie_example.py", line 62, in <module>
            zahnerZennium.setIESecondEdgePotential(10000)

    The error occurred in line **62** of the file **ie_example.py** with the statement ``zahnerZennium.setIESecondEdgePotential(10000)``.
    This means that the value 10000, which should be set, is out of the allowed range.
    """

    message: str

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class TermConnectionError(Exception):
    r"""Term Connection Exception Class

    This exception is thrown when an error occurs with the term communication,
    which has not yet been thrown by a socket exception.

    After this error the connection must be completely rebuilt.
    """

    message: str

    def __init__(self, message: str):
        self.message = message
        super().__init__(self.message)



r"""
  ____       __                        __    __   __      _ __
 /_  / ___ _/ /  ___  ___ ___________ / /__ / /__/ /_____(_) /__
  / /_/ _ `/ _ \/ _ \/ -_) __/___/ -_) / -_)  '_/ __/ __/ /  '_/
 /___/\_,_/_//_/_//_/\__/_/      \__/_/\__/_/\_\\__/_/ /_/_/\_\

Copyright 2024 Zahner-Elektrik GmbH & Co. KG

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

import json
import os
import pathlib
from multiprocessing import Pool

"""
This module contains internally required functions to automatically convert Jupyter notebooks into
pure python source code files. This way the user does not have to manually extract all the python
code from the jupyter code blocks.

Also a function has been implemented to detect if the source code is executed with python or in a
jupyter environment.
"""


def notebookCodeToPython(jupyterNotebookName):
    r"""Convert jupyter-notebook to python.

    This function extracts all code lines from a jupyter notebook and saves them as a file.
    With the standard jupyter export the whole documentation is inserted as a comment, but this is
    not desired.

    The code is then saved to a .py file of equal name.

    :param jupyterNotebookName: The notebook file name.
    """
    notebookText = ""
    os.system(f"python -m black {jupyterNotebookName}")
    print(jupyterNotebookName)
    with open(jupyterNotebookName, "r", encoding="utf-8") as f:
        notebookJson = json.load(f)
        for cell in notebookJson["cells"]:
            if "code" in cell["cell_type"]:
                for line in cell["source"]:
                    notebookText += line.rstrip("\n") + "\n"
                notebookText += "\n"

    jupyterName = jupyterNotebookName.replace("ipynb", "py")
    with open(jupyterName, "wb") as f:
        f.write(notebookText.encode(encoding="UTF-8"))

    os.system(f"python -m black {jupyterName}")
    return


def executionInNotebook():
    r"""Check if the code is executed in jupyter enviroment.

    This function checks if the execution of the code is done in python or in jupyter.
    The recognition is done via IPython and Python, it was only tested that it works on the
    development system with normal Python and the notebooks. It may be that an IPython environment
    is also recognised as a notebook.

    **This function is only needed for automation. The user does not need to use it.**

    :returns: True if the execution takes place in the jupyter enviroment.
    :rtype: bool
    """
    retval = False
    try:
        shell = get_ipython().__class__.__name__
        if "ZMQInteractiveShell" in shell:
            retval = True
    except NameError:
        pass

    return retval


if __name__ == "__main__":
    notebooks = [
        str(file)
        for file in list(
            pathlib.Path(os.path.dirname(os.path.realpath(__file__))).rglob("*.ipynb")
        )
        if "checkpoint" not in str(file)
    ]

    with Pool(max(16, len(notebooks))) as p:
        results = p.map(notebookCodeToPython, notebooks)

    print("finish")



import sys
import time

from delta_remote.connection import DeltaConnection
from delta_remote.script_wrapper import DeltaSCPIWrapper, DeltaSources

from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
from thales_remote.file_interface import ThalesFileInterface
from zahner_analysis.file_import.ism_import import IsmImport

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()
    zahnerZennium.calibrateOffsets()

    fileInterface = ThalesFileInterface("localhost")
    fileInterface.enableKeepReceivedFilesInObject()
    fileInterface.enableAutomaticFileExchange()

deltaConnection = DeltaConnection()
deltaConnection.connect(ip="192.168.2.73", port=8462)

deltaSM18_220 = DeltaSCPIWrapper(deltaConnection)

print(f"IDN:\t{deltaSM18_220.IDN()}")

deltaSM18_220.setProgSourceVoltage(DeltaSources.eth)
deltaSM18_220.setProgSourceCurrent(DeltaSources.eth)

zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
zahnerZennium.setEISNaming("individual")
zahnerZennium.setEISOutputFileName("spectra_5aac")

zahnerZennium.selectPotentiostat(1)
zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
zahnerZennium.setAmplitude(0)
zahnerZennium.setLowerFrequencyLimit(0.01)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(100000)
zahnerZennium.setLowerNumberOfPeriods(5)
zahnerZennium.setLowerStepsPerDecade(5)
zahnerZennium.setUpperNumberOfPeriods(10)
zahnerZennium.setUpperStepsPerDecade(10)
zahnerZennium.setScanDirection("startToMax")
zahnerZennium.setScanStrategy("single")

zahnerZennium.setCurrent(0)
zahnerZennium.enablePotentiostat()

time.sleep(1.0)

deltaSM18_220.setTargetVoltage(6)
deltaSM18_220.setTargetCurrent(20)
deltaSM18_220.enableOutput()

time.sleep(1)

print(f"Measured current\tEL1002:\t\t{zahnerZennium.getCurrent():>10.3f} A")

print(
    f"Measured current\tDelta SM18-220:\t{deltaSM18_220.getMeasuredCurrent():>10.3f} A"
)
print(
    f"Measured voltage\tDelta SM18-220:\t{deltaSM18_220.getMeasuredVoltage():>10.3f} V"
)
print(f"Measured power\t\tDelta SM18-220:\t{deltaSM18_220.getMeasuredPower():>10.3f} W")

zahnerZennium.setAmplitude(5)
zahnerZennium.measureEIS()
zahnerZennium.setAmplitude(0)

deltaSM18_220.disableOutput()

time.sleep(1)

zahnerZennium.disablePotentiostat()

ismFile = IsmImport(fileInterface.getLatestReceivedFile().binaryData)

impedanceFrequencies = ismFile.getFrequencyArray()
impedanceAbsolute = ismFile.getImpedanceArray()
impedancePhase = ismFile.getPhaseArray()
impedanceComplex = ismFile.getComplexImpedanceArray()

figBode, (impedanceAxis) = plt.subplots(1, 1)

phaseAxis = impedanceAxis.twinx()

impedanceAxis.loglog(
    impedanceFrequencies,
    impedanceAbsolute,
    linestyle="dashed",
    linewidth=1,
    marker="o",
    markersize=5,
    fillstyle="none",
    color="blue",
)
impedanceAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
impedanceAxis.yaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
impedanceAxis.set_xlabel(r"f")
impedanceAxis.set_ylabel(r"|Z|")
impedanceAxis.yaxis.label.set_color("blue")
impedanceAxis.grid(which="both", linestyle="dashed", linewidth=0.5)
impedanceAxis.set_xlim(
    [min(impedanceFrequencies) * 0.8, max(impedanceFrequencies) * 1.2]
)

phaseAxis.semilogx(
    impedanceFrequencies,
    np.abs(impedancePhase * (360 / (2 * np.pi))),
    linestyle="dashed",
    linewidth=1,
    marker="o",
    markersize=5,
    fillstyle="none",
    color="red",
)
phaseAxis.yaxis.set_major_formatter(EngFormatter(unit="$°$", sep=""))
phaseAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
phaseAxis.set_xlabel(r"f")
phaseAxis.set_ylabel(r"|Phase|")
phaseAxis.yaxis.label.set_color("red")
phaseAxis.set_ylim([0, 90])
figBode.set_size_inches(15, 6)
plt.show()

zahnerZennium.selectPotentiostat(0)
zenniumConnection.disconnectFromTerm()
fileInterface.close()

deltaSM18_220.setProgSourceVoltage(DeltaSources.front)
deltaSM18_220.setProgSourceCurrent(DeltaSources.front)
deltaConnection.disconnect()

print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import time
import threading

keepThreadRunning = True


def watchThreadFunction():
    global keepThreadRunning

    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "Watch")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)

    while keepThreadRunning:
        time.sleep(1)
        active = zahnerZennium.getTermIsActive()
        print("active state: " + str(active))
        if active:
            print("beat count: " + str(zahnerZennium.getWorkstationHeartBeat()))
    zenniumConnection.disconnectFromTerm()
    return


if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

testThread = threading.Thread(target=watchThreadFunction)
testThread.start()
print("heartbeat thread started")

zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
zahnerZennium.setAmplitude(10e-3)
zahnerZennium.setPotential(0)
zahnerZennium.setLowerFrequencyLimit(500)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(10000)
zahnerZennium.setLowerNumberOfPeriods(5)
zahnerZennium.setLowerStepsPerDecade(2)
zahnerZennium.setUpperNumberOfPeriods(20)
zahnerZennium.setUpperStepsPerDecade(20)
zahnerZennium.setScanDirection("startToMax")
zahnerZennium.setScanStrategy("single")

zahnerZennium.enablePotentiostat()

zahnerZennium.setFrequency(1)
zahnerZennium.setAmplitude(10e-3)
zahnerZennium.setNumberOfPeriods(3)

print("measurement start")
zahnerZennium.measureEIS()
print("measurement end")

zahnerZennium.disablePotentiostat()

print("thread kill")
keepThreadRunning = False
testThread.join()
print("thread killed")

zenniumConnection.disconnectFromTerm()
print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper

from zahner_analysis.file_import.iss_import import IssImport

import matplotlib.pyplot as plt
from matplotlib.ticker import EngFormatter

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

zahnerZennium.setIEOutputPath(r"C:\THALES\temp\ie")

zahnerZennium.setIENaming("individual")

zahnerZennium.calibrateOffsets()

zahnerZennium.setIEFirstEdgePotential(0)
zahnerZennium.setIEFirstEdgePotentialRelation("absolute")
zahnerZennium.setIESecondEdgePotential(0.4)
zahnerZennium.setIESecondEdgePotentialRelation("absolute")
zahnerZennium.setIEThirdEdgePotential(-0.4)
zahnerZennium.setIEThirdEdgePotentialRelation("absolute")
zahnerZennium.setIEFourthEdgePotential(0)
zahnerZennium.setIEFourthEdgePotentialRelation("absolute")

zahnerZennium.setIEPotentialResolution(0.005)
zahnerZennium.setIEMinimumWaitingTime(0.1)
zahnerZennium.setIEMaximumWaitingTime(3)
zahnerZennium.setIERelativeTolerance(0.01)  # 1 %
zahnerZennium.setIEAbsoluteTolerance(0.001)  # 1 mA
zahnerZennium.setIEOhmicDrop(0)

zahnerZennium.setIEScanRate(0.05)
zahnerZennium.setIEMaximumCurrent(3)
zahnerZennium.setIEMinimumCurrent(-3)

zahnerZennium.setIESweepMode("steady state")
zahnerZennium.setIEOutputFileName("ie_steady")

zahnerZennium.checkIESetup()
print(zahnerZennium.readIESetup())

zahnerZennium.measureIE()

zahnerZennium.setIESweepMode("dynamic scan")
zahnerZennium.setIEOutputFileName("ie_dynamic")

zahnerZennium.checkIESetup()
print(zahnerZennium.readIESetup())

zahnerZennium.measureIE()

zahnerZennium.setIESweepMode("fixed sampling")
zahnerZennium.setIEOutputFileName("ie_fixed")

zahnerZennium.checkIESetup()
print(zahnerZennium.readIESetup())

zahnerZennium.measureIE()

zenniumConnection.disconnectFromTerm()

measurementData = IssImport(r"C:\THALES\temp\ie\ie_steady.iss")


fig, (axis) = plt.subplots(1, 1)
axis.semilogy(
    measurementData.getVoltageArray(),
    abs(measurementData.getCurrentArray()),
    color="red",
)

axis.grid(which="both")
axis.xaxis.set_major_formatter(EngFormatter(unit="V"))
axis.yaxis.set_major_formatter(EngFormatter(unit="A"))
axis.set_xlabel(r"Voltage")
axis.set_ylabel(r"Current")
fig.set_size_inches(10, 10)
plt.show()

print("finish")



# simple example script on how to interface with a DE Supply
# communicates via a TCP socket with SCPI commands as found in the programming manual
import socket

SUPPLY_IP = "192.168.2.90"
SUPPLY_PORT = 8462
BUFFER_SIZE = 128  # max msg size
TIMEOUT_SECONDS = 10  # return error if we dont hear from supply within 10 sec
MAX_VOLT = 10  # default
MAX_CUR = 10  # default
validSrcList = [
    "front",
    "web",
    "seq",
    "eth",
    "slot1",
    "slot2",
    "slot3",
    "slot4",
    "loc",
    "rem",
]


supplySocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  # set up socket
supplySocket.connect((SUPPLY_IP, SUPPLY_PORT))  # connect socket
supplySocket.settimeout(TIMEOUT_SECONDS)


def sendAndReceiveCommand(msg):
    msg = msg + "\n"
    supplySocket.sendall(msg.encode("UTF-8"))
    return supplySocket.recv(BUFFER_SIZE).decode("UTF-8").rstrip()


# set value without receiving a response
def sendCommand(msg):
    msg = msg + "\n"
    supplySocket.sendall(msg.encode("UTF-8"))


def setRemoteShutdownState(state):

    if state:
        sendCommand("SYST:RSD 1")
    else:
        sendCommand("SYST:RSD 0")


def setVoltage(volt):
    retval = 0
    if volt > 0 and volt <= MAX_VOLT:
        sendCommand("SOUR:VOLT {0}".format(volt))
    else:

        retval = -1

    return retval


def setCurrent(cur):
    retval = 0
    if cur > 0 and cur <= MAX_VOLT:
        sendCommand("SOUR:CUR {0}".format(cur))
    else:
        retval = -1

    return retval


def readVoltage():
    return sendAndReceiveCommand("SOUR:VOLT?")


def readCurrent():
    return sendAndReceiveCommand("SOUR:CUR?")


def setProgSourceV(src):

    retval = 0
    if src in validSrcList:
        sendCommand("SYST:REM:CV {0}".format(src))

    else:
        retval = -1
    return retval


def setProgSourceI(src):
    retval = 0
    if src.lower() in validSrcList:
        sendCommand("SYST:REM:CC {0}".format(src))
    else:
        retval = -1
    return retval


def setOutputState(state):
    if state:
        sendCommand("OUTPUT 1")

    else:
        sendCommand("OUTPUT 0")


def closeSocket():
    supplySocket.close()


if __name__ == "__main__":
    print(sendAndReceiveCommand("*IDN?"))
    MAX_VOLT = float(sendAndReceiveCommand("SOUR:VOLT:MAX?"))
    MAX_CUR = float(sendAndReceiveCommand("SOUR:CUR:MAX?"))

    print(MAX_VOLT, MAX_CUR)

    setProgSourceV("eth")
    setProgSourceI("eth")

    setVoltage(12.23)
    setCurrent(43.01)

    setRemoteShutdownState(0)  # RSD Enabled = supply off/disabled
    print(readVoltage())
    print(readCurrent())
    closeSocket()



import labview_wrapper as wrap

wrap.init("192.168.2.94")

wrap.transformMethodsToFunctions()

wrap.setPotential(1)
wrap.setCurrent(10e-9)
wrap.setPotentiostatMode(1)  # 1 = pot; 2 = gal
wrap.enablePotentiostat()
print(f"Current: {wrap.getCurrent()}")
print(f"Voltage: {wrap.getVoltage()}")
wrap.disablePotentiostat()

wrap.disconnect()



import os
import jinja2
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import (
    PotentiostatMode,
    ScanDirection,
    ThalesRemoteScriptWrapper,
    FileNaming,
    ScanStrategy,
)


def fillTemplateFile(templateFile: str, outputFile: str, **kwargs):
    """
    Function to fill in template files.

    For example, the following placeholders must be in the passed template:
        `\PYVAR{dc_time}`

    Then the value for the template must be passed as an additional parameter as keyworded variable (kwargs):
        `dc_time=2.0`

    :param templateFile: path of the file containing the templates.
    :param outputFile: path to the file containing the completed templates.
    :param **kwargs: template parameters which should be filled in as keyworded variable.
    """
    latex_jinja_env = jinja2.Environment(
        variable_start_string="\PYVAR{",
        variable_end_string="}",
        trim_blocks=True,
        autoescape=False,
        keep_trailing_newline=True,
        newline_sequence="\r\n",
        loader=jinja2.FileSystemLoader(os.path.abspath(".")),
    )

    template = latex_jinja_env.get_template(templateFile)

    fileString = template.render(**kwargs)

    fileBytes = fileString.encode("utf-8")
    with open(outputFile, "wb") as f:
        f.write(fileBytes)

    return


currentSteps = [0, 0.5, 1, 1.5, 2]
amplitude = 0.05

zenniumConnection = ThalesRemoteConnection()
zenniumConnection.connectToTerm("localhost")
zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
zahnerZennium.forceThalesIntoRemoteScript()

for current in currentSteps:
    print(f"Step: {current}")
    """
    DC Sequence
    """
    fillTemplateFile(
        templateFile=r"ocp_constant_current_template.txt",
        outputFile=r"C:\THALES\script\sequencer\sequences\sequence09.seq",
        ocp_time=10,
        dc_time=20,
        dc_current=current,
    )

    filename = f"{int(current*1000)}ma_current"

    zahnerZennium.disableSequenceAcqGlobal()
    zahnerZennium.setSequenceNaming(FileNaming.INDIVIDUAL)
    zahnerZennium.setSequenceOutputPath(r"C:\THALES\temp")
    zahnerZennium.setSequenceOutputFileName(filename)
    zahnerZennium.selectSequence(9)
    zahnerZennium.runSequence()

    """
    EIS
    """
    zahnerZennium.setEISNaming(FileNaming.INDIVIDUAL)
    zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
    zahnerZennium.setEISOutputFileName(filename)
    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
    zahnerZennium.setAmplitude(amplitude)
    zahnerZennium.setCurrent(current)
    zahnerZennium.setLowerFrequencyLimit(10)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(10000)
    zahnerZennium.setLowerNumberOfPeriods(5)
    zahnerZennium.setLowerStepsPerDecade(2)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(5)
    zahnerZennium.setScanDirection(ScanDirection.START_TO_MAX)
    zahnerZennium.setScanStrategy(ScanStrategy.SINGLE_SINE)

    zahnerZennium.measureEIS()

    zahnerZennium.setAmplitude(0)

zahnerZennium.disablePotentiostat()
zenniumConnection.disconnectFromTerm()



r"""
  ____       __                        __    __   __      _ __
 /_  / ___ _/ /  ___  ___ ___________ / /__ / /__/ /_____(_) /__
  / /_/ _ `/ _ \/ _ \/ -_) __/___/ -_) / -_)  '_/ __/ __/ /  '_/
 /___/\_,_/_//_/_//_/\__/_/      \__/_/\__/_/\_\\__/_/ /_/_/\_\

Copyright 2024 Zahner-Elektrik GmbH & Co. KG

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

import socket
import threading
import queue

from datetime import datetime


class DeltaConnectionError(Exception):
    r"""Delta Connection Exception Class

    This exception is thrown when an error occurs with the term communication,
    which has not yet been thrown by a socket exception.

    After this error the connection must be completely rebuilt.
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class DeltaConnection(object):
    r"""
    Class to handle the connection to the Delta Elektronika power supply connection.
    """

    def __init__(self):
        self.bufferSize = 128
        self.socket_handle = None
        self.receivingWorker = None
        self._receiving_worker_is_running = False

        self.sendMutex = threading.Semaphore(1)
        self.receiveQueue = queue.Queue()

        return

    def connect(self, ip, port):
        r"""Connect to Delta Elektronika power supply.

        :param ip: The hostname or ip-address of the Delta Elektronika power supply.
        :param port: The port.
        :returns: True on success, False if failed.
        :rtype: bool
        """
        self.ip = ip
        self.port = port
        self.socket_handle = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.socket_handle.connect((self.ip, self.port))
        except:
            self.socket_handle = None
            return False

        self._startTelegramListener()

        return True

    def disconnect(self):
        r"""Close the connection."""
        self._stopTelegramListener()
        self._closeSocket()
        self.receiveQueue.put(None)
        return

    def isConnected(self):
        r"""Check if the connection to the Delta Elektronika is open.

        :returns: True if connected, False if not.
        :rtype: bool
        """
        return self.socket_handle != None

    def sendTelegram(self, payload, timeout=None):
        r"""Send a telegram/data.

        :param payload: The actual data which is being sent. This can be a string or an bytearray.
        :param timeout: The timeout for sending data in seconds, blocking at None.
        """
        packet = bytearray()

        if isinstance(payload, str):
            packet += bytearray(payload, "UTF-8")
        else:
            # datatype bytearray
            packet = payload

        if self.sendMutex.acquire(True, timeout=timeout):
            self.socket_handle.settimeout(timeout)
            try:
                self.socket_handle.sendall(packet)
            finally:
                self.socket_handle.settimeout(None)
                self.sendMutex.release()
        else:
            raise DeltaConnectionError("Socket error during data transmission.")
        return

    def waitForBinaryTelegram(self, timeout=None):
        r"""Block infinitely until the next Telegram is arriving.

        If some Telegram has already arrived it will just return the last one from the queue.

        :param timeout: The timeout for sending data in seconds, blocking at None
        :returns: The response from the device or an empty bytearray if someting went wrong.
        :rtype: bytearray
        """
        retval = self.receiveQueue.get(True, timeout=timeout)
        if retval is None:
            raise DeltaConnectionError("Socket error during data reception.")
        return retval

    def waitForStringTelegram(self, timeout=None):
        r"""Block infinitely until the next Telegram is arriving.

        If some Telegram has already arrived it will just return the last one from the queue.

        :param timeout: The timeout for sending data in seconds, blocking at None
        :returns: The last received telegram or an empty string if someting went wrong.
        :rtype: string
        """
        retval = self.waitForBinaryTelegram(timeout).decode("UTF-8")
        return retval

    def sendStringAndWaitForReplyString(self, payload, timeout=None):
        r"""Convenience function: Send a telegram and wait for it's reply.

        If a timeout or a socket error occurs an exception is thrown.

        :param payload: The actual data which is being sent. This can be a string or an bytearray.
        :param timeout: The timeout for sending data in seconds, blocking at None.
        :returns: The last received telegram or an empty string if someting went wrong.
        :rtype: string
        """
        self.sendTelegram(payload, timeout)
        return self.waitForStringTelegram(timeout)

    """
    The following methods should not be called by the user.
    They are marked with the prefix '_' after the Python convention for proteced.
    """

    def _telegramListenerJob(self):
        r"""The method running in a separate thread, pushing the incomming packets into the queues."""
        while self._receiving_worker_is_running:
            telegram = self._readTelegramFromSocket()
            self.receiveQueue.put(telegram)
            if len(telegram) == 0:
                self._receiving_worker_is_running = False
        return

    def _startTelegramListener(self):
        r"""Starts the thread handling the asyncronously incoming data."""
        self._receiving_worker_is_running = True
        self.receivingWorker = threading.Thread(target=self._telegramListenerJob)
        self.receivingWorker.start()
        return

    def _stopTelegramListener(self):
        r"""Stops the thread handling the incoming data gracefully."""
        self._receiving_worker_is_running = False
        self.socket_handle.shutdown(socket.SHUT_RDWR)
        self.receivingWorker.join()
        return

    def _readTelegramFromSocket(self):
        r"""Reads the raw telegram structure from the socket stream.

        When a socket exception occurs, None and an empty byte array are returned.
        The caller of the function then passes the None to the queue to raise an
        exception in the threads waiting at the queue.
        """
        try:
            data = self.socket_handle.recv(self.bufferSize)
        except:
            data = bytearray()
        return data

    def _closeSocket(self):
        r"""Close the socket."""
        self.socket_handle.close()
        self.socket_handle = None
        return



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import (
    PotentiostatMode,
    ThalesRemoteScriptWrapper,
    ScanStrategy,
    ScanDirection,
    FileNaming,
)

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

zahnerZennium.calibrateOffsets()

zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
zahnerZennium.setEISNaming(FileNaming.COUNTER)
zahnerZennium.setEISCounter(13)
zahnerZennium.setEISOutputFileName("spectra")

zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
zahnerZennium.setAmplitude(10e-3)
zahnerZennium.setPotential(1)
zahnerZennium.setLowerFrequencyLimit(10)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(10000)
zahnerZennium.setLowerNumberOfPeriods(5)
zahnerZennium.setLowerStepsPerDecade(2)
zahnerZennium.setUpperNumberOfPeriods(20)
zahnerZennium.setUpperStepsPerDecade(5)
zahnerZennium.setScanDirection(ScanDirection.START_TO_MAX)
zahnerZennium.setScanStrategy(ScanStrategy.SINGLE_SINE)

zahnerZennium.enablePotentiostat()

zahnerZennium.measureEIS()

zahnerZennium.disablePotentiostat()

zahnerZennium.selectPotentiostat(1)

zahnerZennium.setEISNaming(FileNaming.DATE_TIME)
zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test2")
zahnerZennium.setEISOutputFileName("spectra")

zahnerZennium.setAmplitude(50e-3)
zahnerZennium.setScanDirection(ScanDirection.START_TO_MAX)

zahnerZennium.measureEIS()

zahnerZennium.setEISNaming(FileNaming.INDIVIDUAL)
zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test3")

AmplitudesIn_mV_forMeasurement = [5, 10, 20, 50]

for amplitude in AmplitudesIn_mV_forMeasurement:
    zahnerZennium.setEISOutputFileName(f"spectraAmplitude{amplitude}mV")
    zahnerZennium.setAmplitude(amplitude / 1000)
    zahnerZennium.measureEIS()

zahnerZennium.setAmplitude(0)

zahnerZennium.selectPotentiostat(0)

zenniumConnection.disconnectFromTerm()
print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()
    zahnerZennium.calibrateOffsets()

zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

zahnerZennium.setCVOutputFileName("cv_series")
zahnerZennium.setCVNaming("counter")
zahnerZennium.setCVCounter(1)

zahnerZennium.setCVStartPotential(1)
zahnerZennium.setCVUpperReversingPotential(2)
zahnerZennium.setCVLowerReversingPotential(0)
zahnerZennium.setCVEndPotential(1)

zahnerZennium.setCVStartHoldTime(2)
zahnerZennium.setCVEndHoldTime(2)

zahnerZennium.setCVCycles(1.5)
zahnerZennium.setCVSamplesPerCycle(400)
zahnerZennium.setCVScanRate(0.5)

zahnerZennium.setCVMaximumCurrent(0.03)
zahnerZennium.setCVMinimumCurrent(-0.03)

zahnerZennium.setCVOhmicDrop(0)

zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
zahnerZennium.disableCVAnalogFunctionGenerator()

zahnerZennium.checkCVSetup()
print(zahnerZennium.readCVSetup())

zahnerZennium.measureCV()

zahnerZennium.selectPotentiostat(1)

zahnerZennium.setCVNaming("individual")
zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

ScanRatesForMeasurement = [0.1, 0.2, 0.5, 1.0]

for scanRate in ScanRatesForMeasurement:
    zahnerZennium.setCVOutputFileName(
        "cv_scanrate_{:d}mVs".format(int(scanRate * 1000))
    )
    zahnerZennium.setCVScanRate(scanRate)

    zahnerZennium.checkCVSetup()
    print(zahnerZennium.readCVSetup())

    zahnerZennium.measureCV()

zahnerZennium.selectPotentiostat(0)

zenniumConnection.disconnectFromTerm()
print("finish")



import sys
import math
import cmath
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import datetime
import time
import csv
import re

TARGET_HOST = "10.10.255.211"

zenniumConnection = ThalesRemoteConnection()
zenniumConnection.connectToTerm(TARGET_HOST)

zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
zahnerZennium.forceThalesIntoRemoteScript()

print("acq configuration")

acq_setup = zahnerZennium.readAcqSetup()
print(f"setup string from the device:{acq_setup}")

index_channel_name_regex = r"DISP(?P<index>[\d+]);[\d+];(?P<channel_name>.*?);"
headers_match = re.finditer(index_channel_name_regex, acq_setup)

acq_channels = dict()
for match in headers_match:
    acq_channels[int(match.group("index"))] = match.group("channel_name")

print("found channels:")
for index, name in acq_channels.items():
    print(f"index:{index} channel name:{name}")

zahnerZennium.calibrateOffsets()
zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
zahnerZennium.setCurrent(0.1)
zahnerZennium.enablePotentiostat()

duration = 10.0
sampling_frequency = 1.0

interval = 1.0 / sampling_frequency
samples = int(duration * sampling_frequency)
measurements = []
start_time = datetime.datetime.now()

for sample_index in range(samples):
    print(f"sample:{sample_index}/{samples}")
    loop_start = time.time()

    # measure all channels
    current_time = datetime.datetime.now()
    voltage = zahnerZennium.getPotential()
    current = zahnerZennium.getCurrent()
    acq_data = zahnerZennium.readAllAcqChannels()

    # store the results
    measurement = {
        "time": current_time,
        "voltage": voltage,
        "current": current,
        "index": sample_index,
    }

    for key, value in acq_data.items():
        measurement[key] = value

    measurements.append(measurement)

    # calculate the waiting time
    elapsed = time.time() - loop_start
    sleep_time = interval - elapsed

    # wait until the next measurement point
    if sleep_time > 0:
        time.sleep(sleep_time)
    else:
        print("sampling_frequency too high")

with open("polarization.txt", "w", newline="", encoding="utf-8") as f:
    writer = csv.writer(f, delimiter="\t")

    headers = ["time", "voltage", "current"]
    headers.extend([f"{value}" for key, value in acq_channels.items()])
    headers.append("index")

    writer.writerow(headers)

    for m in measurements:
        row = [m["time"].strftime("%Y-%m-%d %H:%M:%S.%f"), m["voltage"], m["current"]]
        for key, value in acq_channels.items():
            row.append(m[key])
        row.append(m["index"])

        writer.writerow(row)


zahnerZennium.disablePotentiostat()
zenniumConnection.disconnectFromTerm()
print("finish")



r"""
  ____       __                        __    __   __      _ __
 /_  / ___ _/ /  ___  ___ ___________ / /__ / /__/ /_____(_) /__
  / /_/ _ `/ _ \/ _ \/ -_) __/___/ -_) / -_)  '_/ __/ __/ /  '_/
 /___/\_,_/_//_/_//_/\__/_/      \__/_/\__/_/\_\\__/_/ /_/_/\_\
 
Copyright 2024 Zahner-Elektrik GmbH & Co. KG
 
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from enum import Enum
import re


class DeltaSCPIError(Exception):
    r"""Delta Connection Exception Class

    This exception is thrown when an error occurs with the communication,
    which has not yet been thrown by a socket exception.

    After this error the connection must be completely rebuilt.
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class DeltaSources(Enum):
    r"""Delta Elektronika power supply source options.

    The class manages the possible sources to be used for setting the target voltage and current.
    """

    front = "front"
    web = "web"
    seq = "seq"
    eth = "eth"
    slot1 = "slot1"
    slot2 = "slot2"
    slot3 = "slot3"
    slot4 = "slot4"
    loc = "loc"
    rem = "rem"


class DeltaSCPIWrapper(object):
    r"""Class containing the SCPI commands for the Delta Elektronika power supply.

    This class wraps the SCPI commands in getter and setter methods.

    :param connection: The connection object to the Delta power supply.
    :type connection: :class:`~delta_remote.connection.DeltaConnection`
    """

    def __init__(self, connection):
        self.defaultTimeout = None
        self.connection = connection

        self.maximumVoltage = self._requestFloatValue("SOUR:VOLT:MAX?")
        self.maximumCurrent = self._requestFloatValue("SOUR:CUR:MAX?")
        return

    def setDefaultTimout(self, time):
        r"""Set the default timeout.

        The command sets the default timeout that is used.

        :param time: Timout in seconds.
        """
        self.defaultTimeout = time
        return

    def IDN(self):
        r"""Querying the device identification.

        This is the standard SCPI command for requesting the identification.

        :returns: The response string from the device.
        """
        return self.executeCommandAndWaitForReply("*IDN?")

    def setTargetVoltage(self, value):
        r"""Set the target output voltage.

        :param value: The output voltage.
        """
        if value >= 0 and value <= self.maximumVoltage:
            self.executeCommand(f"SOUR:VOLT {value}")
        else:
            raise DeltaSCPIError("Voltage out of range.")
        return

    def setTargetCurrent(self, value):
        r"""Set the target output current.

        :param value: The output current.
        """
        if value >= 0 and value <= self.maximumCurrent:
            self.executeCommand(f"SOUR:CUR {value}")
        else:
            raise DeltaSCPIError("Voltage out of range.")
        return

    def getTargetVoltage(self):
        r"""Read the target output voltage.

        :returns: The value.
        """
        return self._requestFloatValue("SOUR:VOLT?")

    def getTargetCurrent(self):
        r"""Read the target output current.

        :returns: The value.
        """
        return self._requestFloatValue("SOUR:CUR?")

    def getMeasuredVoltage(self):
        r"""Measure the actual output voltage.

        :returns: The value.
        """
        return self._requestFloatValue("MEAS:VOLT?")

    def getMeasuredCurrent(self):
        r"""Measure the actual output current.

        :returns: The value.
        """
        return self._requestFloatValue("MEAS:CUR?")

    def getMeasuredPower(self):
        r"""Measure the actual output power.

        :returns: The value.
        """
        return self._requestFloatValue("MEAS:POW?")

    def setProgSourceVoltage(self, source: DeltaSources):
        r"""Set the voltage source option.

        Defines which interface defines the voltage to be applied.

        :param source: The source option.
        :type source: :class:`~delta_remote.script_wrapper.DeltaSources`
        """
        self.executeCommand(f"SYST:REM:CV {source.value}")
        return

    def setProgSourceCurrent(self, source: DeltaSources):
        r"""Set the current source option.

        Defines which interface defines the current to be applied.

        :param source: The source option.
        :type source: :class:`~delta_remote.script_wrapper.DeltaSources`
        """
        self.executeCommand(f"SYST:REM:CC {source.value}")
        return

    def enableOutput(self, enable=True):
        r"""Enable the output of the supply.

        :param enable: True to turn the output on.
        """
        if enable:
            self.executeCommand("OUTPUT 1")
        else:
            self.executeCommand("OUTPUT 0")
        return

    def disableOutput(self):
        r"""Disable the output of the supply."""
        self.enableOutput(False)
        return

    def executeCommand(self, command, timeout=None):
        r"""Executes a command.

        :param command: The command.
        :param timeout: The timeout for sending data in seconds, blocking at None.
        """
        if timeout is None:
            timeout = self.defaultTimeout
        return self.connection.sendTelegram(command + "\n", timeout)

    def executeCommandAndWaitForReply(self, command, timeout=None):
        r"""Executes a command and waits for the reply from the device.

        :param command: The command.
        :param timeout: The timeout for sending data in seconds, blocking at None.
        """
        if timeout is None:
            timeout = self.defaultTimeout
        return self.connection.sendStringAndWaitForReplyString(
            command + "\n", timeout
        ).rstrip()

    def _requestValueAndParseUsingRegexp(self, command, pattern):
        reply = self.executeCommandAndWaitForReply(command)
        match = re.search(pattern, reply)
        return float(match.group(1))

    def _requestFloatValue(self, command):
        return float(self.executeCommandAndWaitForReply(command))



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper

from zahner_analysis.file_import.ism_import import IsmImport
from zahner_analysis.plotting.impedance_plot import nyquistPlotter, bodePlotter

import matplotlib.pyplot as plt
import numpy as np

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

zahnerZennium.setEISNaming("counter")
zahnerZennium.setEISCounter(1)
zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
zahnerZennium.setEISOutputFileName("spectra")

zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
zahnerZennium.setAmplitude(10e-3)
zahnerZennium.setPotential(0)
zahnerZennium.setLowerFrequencyLimit(0.01)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(200000)
zahnerZennium.setLowerNumberOfPeriods(3)
zahnerZennium.setLowerStepsPerDecade(5)
zahnerZennium.setUpperNumberOfPeriods(20)
zahnerZennium.setUpperStepsPerDecade(10)
zahnerZennium.setScanDirection("startToMax")
zahnerZennium.setScanStrategy("single")

zahnerZennium.enablePotentiostat()
zahnerZennium.measureEIS()
zahnerZennium.disablePotentiostat()

zahnerZennium.setAmplitude(0)

zenniumConnection.disconnectFromTerm()

ismFile = IsmImport(r"C:\THALES\temp\test1\spectra_0001.ism")

impedanceFrequencies = ismFile.getFrequencyArray()

impedanceAbsolute = ismFile.getImpedanceArray()
impedancePhase = ismFile.getPhaseArray()

impedanceComplex = ismFile.getComplexImpedanceArray()

print("Measurement end time: " + str(ismFile.getMeasurementEndDateTime()))

(figNyquist, nyquistAxis) = nyquistPlotter(impedanceObject=ismFile)

figNyquist.suptitle("Nyquist")
figNyquist.set_size_inches(18, 18)

plt.show()

figNyquist.savefig("nyquist.svg")

(figBode, (impedanceAxis, phaseAxis)) = bodePlotter(impedanceObject=ismFile)

figBode.suptitle("Bode")
figBode.set_size_inches(18, 12)

plt.show()

figBode.savefig("bode.svg")



import socket


class BCMuxInterface:
    r"""BC-Mux control class.

    With this class the `BC-MUX Multiplexer <https://zahner.de/products-details/multiplexer/bc-mux>`_ can
    be controlled remotely without the BC-Mux Controller software.

    The USB interface of the BC Mux is not supported by Python. Also the network settings must be
    done with the program BC-Mux Network Config before using python.

    The BC-MUX is an extension which makes it possible to separate up to 16 channels of a cyclizer
    individually with switch boxes from the cyclizer and to switch them to the Zennium for e.g.
    impedance measurements. This allows the cyclizer to be extended up to 16 channels with sequential
    impedance measurements. Only one channel at a time can be switched to the Zennium for a measurement,
    no parallel impedance measurements are possible.

    This class makes it possible to control the Zennium and the Multiplexer from one Python instance
    via Remote2, which makes the use more flexible than with the BC-Mux Controller.
    Also, if the cyclizer supports it, the complete system can be controlled from a single Python software.

    :param ip: SerialCommandInterface object to control the device.
    :type ip: str
    :param port: SerialDataInterface object for online data.
    :type port: int
    """

    BUFFER_SIZE = 1024

    def __init__(self, ip, port):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.ip = ip
        self.port = port
        self.socket.connect((self.ip, self.port))
        return

    def close(self):
        r"""Closing the connection.

        Disconnects the TCP/IP connection to the BC-MUX.
        """
        self.socket.close()
        return

    def connectChannel(self, channel):
        r"""Connects the channel to the zennium.

        With this command, a channel is disconnected from the cyclizer and switched to the Zennium,
        for example for impedance measurements.

        :param channel: The channel to connect to the zennium.
        :returns: The response string from the device.
        :rtype: string
        """
        command = "ch {}"
        return self._executeCommandAndReadReply(command.format(channel))

    def disconnectChannel(self):
        r"""Disconnects all channels from the zennium.

        All channels are disconnected from the Zennium and switched to the specific cyclizer channel.

        :returns: The response string from the device.
        :rtype: string
        """
        command = "ch 0"
        return self._executeCommandAndReadReply(command)

    def setPulseLength(self, length):
        r"""Setting the relais control.

        The BC-MUX supports switchboxes containing monostable or bistable relais. With this command,
        the control of the relais is set.

        If a number other than 0 is set, the relay is switched with a pulse.
        The pulse is then the number in 10 milliseconds long.

        :param length: The length of the switching pulse in 10 milliseconds. 0 for monostable relays. 1 for 10 ms. Maximum 255.
        :returns: The response string from the device.
        :rtype: string
        """
        command = "puls {}"
        return self._executeCommandAndReadReply(command.format(length))

    def _executeCommandAndReadReply(self, command):
        r"""Private function to send a command to the device and read a string.

        This command sends the command to the device and returns the response from the device.

        :returns: Response string from the device.
        :rtype: string
        """
        command += "\r\n"
        self.socket.send(command.encode("utf-8"))
        data = self.socket.recv(BCMuxInterface.BUFFER_SIZE)
        return data.decode("utf-8")


if __name__ == "__main__":
    TCP_IP = "169.169.169.169"
    TCP_PORT = 4223

    bcMux = BCMuxInterface(TCP_IP, TCP_PORT)

    bcMux.setPulseLength(250)
    bcMux.disconnectChannel()

    for i in range(16):
        print(f"Channel: {i+1}")
        bcMux.connectChannel(i + 1)
        bcMux.disconnectChannel()

    bcMux.close()

    print("finish")



from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import time

zenniumConnection = ThalesRemoteConnection()
zenniumConnection.connectToTerm("192.168.2.47", "ScriptRemote")

zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
zahnerZennium.forceThalesIntoRemoteScript()

zahnerZennium.enableFraMode()

zahnerZennium.setFraVoltageMinimum(0)
zahnerZennium.setFraVoltageMaximum(18)
zahnerZennium.setFraCurrentMinimum(0)
zahnerZennium.setFraCurrentMaximum(220)

zahnerZennium.setFraVoltageInputGain(5 * (4 / 5))  # gain * (4 / 5)
zahnerZennium.setFraVoltageInputOffset(-25e-3)

zahnerZennium.setFraVoltageOutputGain(18.0 / 5.0)
zahnerZennium.setFraVoltageOutputOffset(-0.3)

zahnerZennium.setFraCurrentInputGain(-242)
zahnerZennium.setFraCurrentInputOffset(-15.03)

zahnerZennium.setFraCurrentOutputGain(-220.0 / 5.0)
zahnerZennium.setFraCurrentOutputOffset(-3)

zahnerZennium.setFraPotentiostatMode(PotentiostatMode.POTMODE_GALVANOSTATIC)
zahnerZennium.setCurrent(0)

for i in range(10):
    zahnerZennium.setCurrent(i)
    time.sleep(0.5)
    print(f"Potential:\t{zahnerZennium.getPotential()}\tV")
    print(f"Current:\t{zahnerZennium.getCurrent()}\tA")

zahnerZennium.setEISOutputPath(r"C:\THALES\temp\test1")
zahnerZennium.setEISNaming("counter")
zahnerZennium.setEISCounter(13)
zahnerZennium.setEISOutputFileName("spectra")

zahnerZennium.setCurrent(10)
zahnerZennium.setAmplitude(2)
zahnerZennium.setLowerFrequencyLimit(1)
zahnerZennium.setStartFrequency(1000)
zahnerZennium.setUpperFrequencyLimit(10e3)
zahnerZennium.setLowerNumberOfPeriods(3)
zahnerZennium.setLowerStepsPerDecade(2)
zahnerZennium.setUpperNumberOfPeriods(20)
zahnerZennium.setUpperStepsPerDecade(3)
zahnerZennium.setScanDirection("startToMax")
zahnerZennium.setScanStrategy("single")

zahnerZennium.measureEIS()

zenniumConnection.disconnectFromTerm()
print("finish")



from thales_remote.epc_scpi_handler import EpcScpiHandlerFactory, EpcScpiHandler
from thales_remote.script_wrapper import PotentiostatMode
from zahner_potentiostat.scpi_control.datahandler import DataManager
from zahner_potentiostat.display.onlinedisplay import OnlineDisplay
from zahner_potentiostat.scpi_control.datareceiver import TrackTypes


class TargetCurrents:
    def __init__(self, dc, amplitude, scanrate):
        self.dc = dc
        self.amplitude = amplitude
        self.scanrate = scanrate
        return


def measure_UI_EPC(deviceHandler):
    for _ in range(3):
        print(
            f"EPC-Potential:\t{deviceHandler.sharedZenniumInterface.getPotential():>10.6f} V"
        )
        print(
            f"EPC-Current:\t{deviceHandler.sharedZenniumInterface.getCurrent():>10.3e} A"
        )
    return


def measure_UI_SCPI(deviceHandler):
    for _ in range(3):
        print(f"SCPI-Potential:\t{deviceHandler.scpiInterface.getPotential():>10.6f} V")
        print(f"SCPI-Current:\t{deviceHandler.scpiInterface.getCurrent():>10.3e} A")
    return


if __name__ == "__main__":
    startCurrent = 0.0
    handlerFactory = EpcScpiHandlerFactory("192.168.2.94")
    deviceHandler = handlerFactory.createEpcScpiHandler(
        epcChannel=1, serialNumber=33021
    )

    deviceHandler.scpiInterface.calibrateOffsets()

    deviceHandler.acquireSharedZennium(blocking=True)
    deviceHandler.switchToEPC()

    deviceHandler.sharedZenniumInterface.calibrateOffsets()

    deviceHandler.sharedZenniumInterface.setPotentiostatMode(
        PotentiostatMode.POTMODE_GALVANOSTATIC
    )
    deviceHandler.sharedZenniumInterface.setCurrent(startCurrent)
    deviceHandler.sharedZenniumInterface.enablePotentiostat()

    measure_UI_EPC(deviceHandler)
    deviceHandler.switchToSCPIAndReleaseSharedZennium(keepPotentiostatState=True)
    measure_UI_SCPI(deviceHandler)

    measurementSettings = [
        TargetCurrents(dc=1, amplitude=0.1, scanrate=0.1),
        TargetCurrents(dc=2, amplitude=0.1, scanrate=0.1),
        TargetCurrents(dc=4, amplitude=0.2, scanrate=0.5),
    ]

    for setting in measurementSettings:
        deviceHandler.scpiInterface.setCurrentValue(
            deviceHandler.scpiInterface.getCurrent()
        )

        configuration = {
            "figureTitle": f"Online Display Targetcurrent: {setting.dc} Scanrate: {setting.scanrate}",
            "xAxisLabel": "Time",
            "xAxisUnit": "s",
            "xTrackName": TrackTypes.TIME.toString(),
            "yAxis": [
                {
                    "label": "Voltage",
                    "unit": "V",
                    "trackName": TrackTypes.VOLTAGE.toString(),
                },
                {
                    "label": "Current",
                    "unit": "A",
                    "trackName": TrackTypes.CURRENT.toString(),
                },
            ],
        }
        onlineDisplay = OnlineDisplay(
            deviceHandler.scpiInterface.getDataReceiver(),
            displayConfiguration=configuration,
        )

        deviceHandler.scpiInterface.setCurrentParameter(setting.dc)
        deviceHandler.scpiInterface.setScanRateParameter(setting.scanrate)
        deviceHandler.scpiInterface.measureRampValueInScanRate()

        deviceHandler.scpiInterface.setMaximumTimeParameter(5)
        deviceHandler.scpiInterface.measurePolarization()
        while deviceHandler.acquireSharedZennium(blocking=False) == False:
            deviceHandler.scpiInterface.measurePolarization()

        dataManager = DataManager(deviceHandler.scpiInterface.getDataReceiver())
        dataManager.saveDataAsText(f"ramp_to{setting.dc}a_{setting.scanrate}apers.txt")

        onlineDisplay.close()
        del onlineDisplay
        del dataManager

        measure_UI_SCPI(deviceHandler)
        deviceHandler.switchToEPC(keepPotentiostatState=True)
        measure_UI_EPC(deviceHandler)

        deviceHandler.sharedZenniumInterface.setEISNaming("individual")
        deviceHandler.sharedZenniumInterface.setEISOutputPath(r"C:\THALES\temp")
        deviceHandler.sharedZenniumInterface.setEISOutputFileName(
            f"{setting.dc}adc_{setting.amplitude}aac".replace(".", "")
        )

        deviceHandler.sharedZenniumInterface.setPotentiostatMode(
            PotentiostatMode.POTMODE_GALVANOSTATIC
        )
        deviceHandler.sharedZenniumInterface.setPotential(setting.dc)
        deviceHandler.sharedZenniumInterface.setAmplitude(setting.amplitude)

        deviceHandler.sharedZenniumInterface.setLowerFrequencyLimit(100)
        deviceHandler.sharedZenniumInterface.setStartFrequency(250)
        deviceHandler.sharedZenniumInterface.setUpperFrequencyLimit(500)
        deviceHandler.sharedZenniumInterface.setLowerNumberOfPeriods(5)
        deviceHandler.sharedZenniumInterface.setLowerStepsPerDecade(10)
        deviceHandler.sharedZenniumInterface.setUpperNumberOfPeriods(20)
        deviceHandler.sharedZenniumInterface.setUpperStepsPerDecade(10)
        deviceHandler.sharedZenniumInterface.setScanDirection("startToMax")
        deviceHandler.sharedZenniumInterface.setScanStrategy("single")

        deviceHandler.sharedZenniumInterface.enablePotentiostat()
        deviceHandler.sharedZenniumInterface.measureEIS()
        deviceHandler.sharedZenniumInterface.setAmplitude(0)

        measure_UI_EPC(deviceHandler)
        deviceHandler.switchToSCPIAndReleaseSharedZennium(keepPotentiostatState=True)
        measure_UI_SCPI(deviceHandler)

    deviceHandler.scpiInterface.setPotentiostatEnabled(False)
    handlerFactory.closeAll()
    print("finish")
